type warning =
	| WAll
	| WInternal
	| WTemp
	| WInfo
	| WUser
	| WCompiler
	| WParser
	| WTyper
	| WMacro
	| WOptimizer
	| WGenerator
	| WDeprecated
	| WDeprecatedDefine
	| WDeprecatedEnumAbstract
	| WVarInit
	| WVarShadow
	| WExternWithExpr
	| WStaticInitOrder
	| WClosureCompare
	| WReservedTypePath
	| WPatternMatcher
	| WUnusedPattern
	| WConstructorInliningCancelled

type warning_obj = {
	w_name : string;
	w_doc : string;
	w_generic : bool;
	w_parent : warning option;
}

let warning_obj = function
	| WAll -> {w_name = "WAll"; w_doc = "The mother of all warnings"; w_generic = true; w_parent = None}
	| WInternal -> {w_name = "WInternal"; w_doc = "Reserved for internal use"; w_generic = true; w_parent = Some WAll}
	| WTemp -> {w_name = "WTemp"; w_doc = "Reserved for internal use"; w_generic = true; w_parent = Some WAll}
	| WInfo -> {w_name = "WInfo"; w_doc = "Generic information like $type"; w_generic = true; w_parent = Some WAll}
	| WUser -> {w_name = "WUser"; w_doc = "Custom user warnings, e.g. from Context.warning"; w_generic = true; w_parent = Some WAll}
	| WCompiler -> {w_name = "WCompiler"; w_doc = "Warnings from the compiler frontend, e.g. argument parsing"; w_generic = true; w_parent = Some WAll}
	| WParser -> {w_name = "WParser"; w_doc = "Warnings related to lexing and parsing"; w_generic = true; w_parent = Some WAll}
	| WTyper -> {w_name = "WTyper"; w_doc = "Warnings from the typer"; w_generic = true; w_parent = Some WAll}
	| WMacro -> {w_name = "WMacro"; w_doc = "Warning related to macros"; w_generic = true; w_parent = Some WAll}
	| WOptimizer -> {w_name = "WOptimizer"; w_doc = "Warnings related to optimization"; w_generic = true; w_parent = Some WAll}
	| WGenerator -> {w_name = "WGenerator"; w_doc = "Warnings related to code generation"; w_generic = true; w_parent = Some WAll}
	| WDeprecated -> {w_name = "WDeprecated"; w_doc = "This is deprecated and should no longer be used"; w_generic = false; w_parent = Some WAll}
	| WDeprecatedDefine -> {w_name = "WDeprecatedDefine"; w_doc = "This define is deprecated and should no longer be used"; w_generic = false; w_parent = Some WDeprecated}
	| WDeprecatedEnumAbstract -> {w_name = "WDeprecatedEnumAbstract"; w_doc = "`@:enum abstract` is deprecated, `enum abstract` should be used instead"; w_generic = false; w_parent = Some WDeprecated}
	| WVarInit -> {w_name = "WVarInit"; w_doc = "A local variable might be used before being assigned a value"; w_generic = false; w_parent = Some WTyper}
	| WVarShadow -> {w_name = "WVarShadow"; w_doc = "A local variable hides another by using the same name"; w_generic = false; w_parent = Some WTyper}
	| WExternWithExpr -> {w_name = "WExternWithExpr"; w_doc = "A non-inline field marked as extern has an expression"; w_generic = false; w_parent = Some WTyper}
	| WStaticInitOrder -> {w_name = "WStaticInitOrder"; w_doc = "The compiler could not determine a type order due to mutually dependent static initializations"; w_generic = false; w_parent = Some WTyper}
	| WClosureCompare -> {w_name = "WClosureCompare"; w_doc = "Closures are being compared, which might be undefined"; w_generic = false; w_parent = Some WTyper}
	| WReservedTypePath -> {w_name = "WReservedTypePath"; w_doc = "A type path is being used that is supposed to be reserved on the current target"; w_generic = false; w_parent = Some WTyper}
	| WPatternMatcher -> {w_name = "WPatternMatcher"; w_doc = "Warnings related to the pattern matcher"; w_generic = false; w_parent = Some WTyper}
	| WUnusedPattern -> {w_name = "WUnusedPattern"; w_doc = "The pattern is not used because previous cases cover"; w_generic = false; w_parent = Some WPatternMatcher}
	| WConstructorInliningCancelled -> {w_name = "WConstructorInliningCancelled"; w_doc = "Constructor call could not be inlined because a field is uninitialized"; w_generic = false; w_parent = Some WTyper}

let from_string = function
	| "WAll" -> WAll
	| "WInternal" -> WInternal
	| "WTemp" -> WTemp
	| "WInfo" -> WInfo
	| "WUser" -> WUser
	| "WCompiler" -> WCompiler
	| "WParser" -> WParser
	| "WTyper" -> WTyper
	| "WMacro" -> WMacro
	| "WOptimizer" -> WOptimizer
	| "WGenerator" -> WGenerator
	| "WDeprecated" -> WDeprecated
	| "WDeprecatedDefine" -> WDeprecatedDefine
	| "WDeprecatedEnumAbstract" -> WDeprecatedEnumAbstract
	| "WVarInit" -> WVarInit
	| "WVarShadow" -> WVarShadow
	| "WExternWithExpr" -> WExternWithExpr
	| "WStaticInitOrder" -> WStaticInitOrder
	| "WClosureCompare" -> WClosureCompare
	| "WReservedTypePath" -> WReservedTypePath
	| "WPatternMatcher" -> WPatternMatcher
	| "WUnusedPattern" -> WUnusedPattern
	| "WConstructorInliningCancelled" -> WConstructorInliningCancelled
	| _ -> raise Exit
